from packaging.specifiers import SpecifierSet
from packaging.version import Version

if __name__ == "__main__":
    # More info: https://packaging.pypa.io/en/latest/specifiers.html

    # Requests 2.26.0 (current version)
    # FastAPI: "requests >=2.24.0,<3.0.0"

    requests_versions = (
        "2.23.0",
        "2.23.1",
        "2.24.0",
        "2.24.1",
        "2.25.0",
        "2.25.1",
        "2.26.0",
        "2.26.1",
        "2.27.0",
        "2.27.1",
        "2.32.0",
        "2.32.1",
        "3.0.0",
    )

    cleo_versions = (
        "0.7.0",
        "0.7.1",
        "0.8.0",
        "0.8.1",
        "0.9.0",
        "0.9.1",
        "1.0.0a1",
        "1.0.0a2",
        "1.0.0",
        "1.0.1",
        "1.1.0",
        "2.0.0",
    )

    # Requests
    spec_fastapi = SpecifierSet(">=2.24.0") & SpecifierSet("<3.0.0")
    spec_half_fastapi = SpecifierSet(">=2.24.0")
    spec_tilde_fastapi = SpecifierSet("~=2.24")

    # Less conservative
    spec_nome = SpecifierSet(">=2.26.0") & SpecifierSet("<3.0.0")
    # More conservative
    spec_tilde_nome = SpecifierSet("~=2.26.0")

    # Cleo
    spec_cleo = SpecifierSet(">=0.8.1") & SpecifierSet("<1.0.0")
    spec_half_cleo = SpecifierSet(">=0.8.1")
    spec_tilde_cleo = SpecifierSet("~=0.8.1")

    # Results
    # Requests
    print(spec_fastapi)
    for version in requests_versions:
        print(f"{version:<6}: {Version(version) in spec_fastapi}")

    print(f"\n{spec_half_fastapi}")
    for version in requests_versions:
        print(f"{version:<6}: {Version(version) in spec_half_fastapi}")

    print(f"\n{spec_tilde_fastapi}")
    for version in requests_versions:
        print(f"{version:<6}: {Version(version) in spec_tilde_fastapi}")

    print(f"\n{spec_nome}")
    for version in requests_versions:
        print(f"{version:<6}: {Version(version) in spec_nome}")

    print(f"\n{spec_tilde_nome}")
    for version in requests_versions:
        print(f"{version:<6}: {Version(version) in spec_tilde_nome}")

    # Cleo
    print(f"\n{spec_cleo}")
    for version in cleo_versions:
        print(f"{version:<6}: {Version(version) in spec_cleo}")

    print(f"\n{spec_half_cleo}")
    for version in cleo_versions:
        print(f"{version:<6}: {Version(version) in spec_half_cleo}")

    print(f"\n{spec_tilde_cleo}")
    for version in cleo_versions:
        print(f"{version:<6}: {Version(version) in spec_tilde_cleo}")



from http import HTTPStatus

import requests
from cleo import Command
from clikit.api.io import flags

from .constants import (
    AVAILABLE_MSG,
    HTTP_STATUS_CODE_MSG,
    NOT_AVAILABLE_MSG,
    PYPI_BASE_URL,
)


class PypiCommand(Command):
    """
    Check the availability of a package name in PyPI

    pypi
        {name : What package name do you want to see if it's available?}
    """

    def handle(self):
        name = self.argument("name")
        url = f"{PYPI_BASE_URL}{name}"

        with requests.Session() as s:
            r = s.get(url)

            status_code = r.status_code
            updated_url = r.url

        # More info:
        # - https://stackoverflow.com/a/29880372
        # - https://github.com/python/cpython/blob/3.6/Lib/http/__init__.py
        # - https://developer.mozilla.org/en-US/docs/Web/HTTP/Status
        status_code_description = HTTPStatus(status_code).phrase
        # or
        # status_code_description = HTTPStatus(status_code).description

        # Verbosity:
        # - https://github.com/sdispater/clikit/blob/0.6.2/src/clikit/api/io/flags.py
        # - https://github.com/sdispater/cleo/issues/49
        # - https://github.com/sdispater/cleo/blob/0.8.1/cleo/io/io_mixin.py
        # - https://github.com/sdispater/cleo/blob/0.8.1/cleo/commands/command.py#L224
        self.line(
            HTTP_STATUS_CODE_MSG.format(code=status_code, desc=status_code_description),
            verbosity=flags.VERBOSE,  # -v flag
        )

        is_available = status_code == 404  # Not Found

        if is_available:
            self.line(AVAILABLE_MSG.format(name=name))
        else:
            self.line(NOT_AVAILABLE_MSG.format(name=name, url=updated_url))



from cleo import Application
from cleo.config import ApplicationConfig

from . import __name__, __version__
from .npm_command import NpmCommand
from .pypi_command import PypiCommand

# Single command applications need a workaround:
# - https://github.com/sdispater/cleo/issues/71
# - https://github.com/sdispater/clikit/blob/0.6.2/src/clikit/config/default_application_config.py#L66
# - https://github.com/sdispater/clikit/blob/0.6.2/src/clikit/api/config/application_config.py
# - https://github.com/sdispater/clikit/blob/0.6.2/src/clikit/console_application.py
# - https://github.com/sdispater/cleo/issues/66
# - https://cleo.readthedocs.io/en/latest/single_command_tool.html
# The future version 1.0.0 seems to add native support for this:
# - https://github.com/sdispater/cleo/blob/1.0.0a4/cleo/application.py


# More info:
# - https://github.com/sdispater/clikit/blob/0.6.2/src/clikit/api/config/application_config.py#L257
# - https://github.com/sdispater/clikit/blob/0.6.2/src/clikit/api/config/application_config.py#L76
config = ApplicationConfig(name=__name__, version=__version__)
config.set_display_name(__name__)

# More info:
# - https://github.com/sdispater/cleo/blob/0.8.1/cleo/application.py
application = Application(config=config, complete=False)

application.add(NpmCommand())
application.add(PypiCommand())

# application.run()



PYPI_BASE_URL: str = "https://pypi.org/project/"
NPM_BASE_URL: str = "https://www.npmjs.com/package/"

HTTP_STATUS_CODE_MSG: str = "<comment>HTTP response status code: {code} ({desc})</comment>"
AVAILABLE_MSG: str = "<fg=green;options=bold>{name}</> is available!"
NOT_AVAILABLE_MSG: str = "<fg=red;options=bold>{name}</> ({url}) is not available..."



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['nome']

package_data = \
{'': ['*']}

install_requires = \
['requests ~=2.26.0', 'cleo ~=0.8.1']

entry_points = \
{'console_scripts': ['nome = nome.application:application.run']}

setup(name='nome',
      version='0.1.0',
      description='A Python CLI to check if a package name is available in PyPI or npm.',
      author='João Palmeiro',
      author_email='joaommpalmeiro@gmail.com',
      url='https://github.com/joaopalmeiro/nome',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6,<4',
     )



from http import HTTPStatus

import requests
from cleo import Command
from clikit.api.io import flags

from .constants import (
    AVAILABLE_MSG,
    HTTP_STATUS_CODE_MSG,
    NOT_AVAILABLE_MSG,
    NPM_BASE_URL,
)


class NpmCommand(Command):
    """
    Check the availability of a package name in npm

    npm
        {name : What package name do you want to see if it's available?}
    """

    def handle(self):
        name = self.argument("name")
        url = f"{NPM_BASE_URL}{name}"

        with requests.Session() as s:
            r = s.get(url)

            status_code = r.status_code
            updated_url = r.url

        status_code_description = HTTPStatus(status_code).phrase

        self.line(
            HTTP_STATUS_CODE_MSG.format(code=status_code, desc=status_code_description),
            verbosity=flags.VERBOSE,
        )

        is_available = status_code == 404

        if is_available:
            self.line(AVAILABLE_MSG.format(name=name))
        else:
            self.line(NOT_AVAILABLE_MSG.format(name=name, url=updated_url))
